@@ -7,7 +7,7 @@
 #'
 #' str(all_patterns)
 all_patterns = list(
-  `rnw` = list(chunk.begin = '^<<(.*)>>=', chunk.end = '^@\\s*%*',
+  `rnw` = list(chunk.begin = '^\\s*<<(.*)>>=', chunk.end = '^\\s*@\\s*%*',
                inline.code = '\\\\Sexpr\\{([^}]*)\\}',
                input.doc = '(^|\n) *\\\\SweaveInput\\{([^}]*)\\}',
                inline.comment = '^\\s*%.*',
@@ -19,8 +19,8 @@ all_patterns = list(
 
   `brew` = list(inline.code = '<%[=]{0,1}\\s+([^%]*)\\s+[-]*%>'),
 
-  `tex` = list(chunk.begin = '^%+\\s*begin.rcode\\s*(.*)',
-               chunk.end = '^%+\\s*end.rcode', chunk.code = '^%+',
+  `tex` = list(chunk.begin = '^\\s*%+\\s*begin.rcode\\s*(.*)',
+               chunk.end = '^\\s*%+\\s*end.rcode', chunk.code = '^%+',
                ref.chunk = '^%+\\s*<<(.*)>>\\s*$',
                inline.comment = '^\\s*%.*',
                global.options = '%+\\s*roptions\\s*([^\n]*)',
@@ -29,7 +29,7 @@ all_patterns = list(
                document.begin = '\n*\\s*\\\\begin\\{document\\}',
                ref.label = '^## @knitr (.*)$'),
 
-  `html` = list(chunk.begin = '^<!--\\s*begin.rcode\\s*(.*)',
+  `html` = list(chunk.begin = '^\\s*<!--\\s*begin.rcode\\s*(.*)',
                 chunk.end = '^\\s*end.rcode\\s*-->',
                 ref.chunk = '^\\s*<<(.*)>>\\s*$',
                 inline.code = '<!--\\s*rinline\\s*([^>]*)\\s*-->',
@@ -37,15 +37,15 @@ all_patterns = list(
                 header.begin = '\n*\\s*<head>',
                 ref.label = '^## @knitr (.*)$'),
 
-  `md` = list(chunk.begin = '^`{3,}\\s*\\{r(.*)\\}\\s*$',
-              chunk.end = '^`{3,}\\s*$',
+  `md` = list(chunk.begin = '^\\s*`{3,}\\s*\\{r(.*)\\}\\s*$',
+              chunk.end = '^\\s*`{3,}\\s*$',
               ref.chunk = '^\\s*<<(.*)>>\\s*$',
               inline.code = '`r +([^`\n]+)\\s*`',
               global.options = '`ro\\s+([^`]*)\\s+or`',
               ref.label = '^## @knitr (.*)$'),
 
-  `rst` = list(chunk.begin = "^\\.{2}\\s+\\{r(.*)\\}\\s*$",
-               chunk.end = "^\\.{2}\\s+\\.{2,}\\s*$",
+  `rst` = list(chunk.begin = "^\\s*\\.{2}\\s+\\{r(.*)\\}\\s*$",
+               chunk.end = "^\\s*\\.{2}\\s+\\.{2,}\\s*$",
                chunk.code = "^\\.{2}",
                ref.chunk = "^\\.*\\s*<<(.*)>>\\s*$",
                inline.code = ":r:`([^`]*)`",
@@ -132,16 +132,21 @@ group_pattern = function(pattern) {
 
 ## automatically detect the chunk patterns
 detect_pattern = function(text, ext) {
+  if (!missing(ext)) {
+    if (ext %in% c('rnw', 'snw', 'stex')) return('rnw')
+    if (ext == 'brew') return('brew')
+    if (ext %in% c('htm', 'html', 'rhtm', 'rhtml')) return('html')
+    if (ext %in% c('rmd', 'rmarkdown', 'markdown', 'md')) return('md')
+    if (ext %in% c('rst', 'rrst')) return('rst')
+  }
   for (p in names(all_patterns)) {
     for (i in c('chunk.begin', 'inline.code')) {
       pat = all_patterns[[p]][[i]]
       if (length(pat) && length(grep(pat, text))) return(p)
     }
   }
-  if (missing(ext)) return(NULL)
-  if (ext %in% c('rnw', 'snw', 'stex', 'rtex')) return('rnw')
-  if (ext %in% c('htm', 'rhtm', 'rhtml')) return('html')
-  if (ext %in% c('rmd', 'rmarkdown', 'markdown')) return('md')
-  if (ext == 'rrst') return('rst')
+  # *.Rtex indicates the tex syntax in knitr, but Rnw syntax in traditional
+  # Sweave, which should have been detected in the above loop
+  if (!missing(ext) && ext == 'rtex') return('rnw')
   NULL
 }
